# Mathematical for Planet Earth Competition 2017

# David Niyukuri
# PhD student in Epidemiology
# Stellenbosch University
# South African Centre of Excellence in Epidemiological Modelling and Analysis (SACEMA)
# Faculty of Medicine and Health Sciences


# Theme: Origin of life and evolution: phylogenetic tree

# Subtheme: Transmission network of an infectious disease (HIV & flu) from a phylogenetic tree


# Clean up
rm(list = ls())

setwd("/home/david/Dropbox/MPE_2017/")

pacman::p_load("igraph", "expoTree", "phylosim", "reader", "network")

# 1. Simulate an epidemic with expoTree and a transmission network
##################################################################

transm.ls <- sim.epi(6,1,0.1,0.4,6) 


# build a transmission tree which guides the sequence simulation
transm.tree <- epi2tree(transm.ls)


# plot(transm.tree)

# extract transmission network

keeps <- c("id", "parent")

transm.graph.dat <- transm.ls[keeps]

transm.df <- cbind(transm.graph.dat$id, transm.graph.dat$parent)


transm.graph <- as.matrix(transm.df)

transm.graph[,1] <- as.character(transm.graph[,1]) # donors
transm.graph[,2] <- as.character(transm.graph[,2]) # recipients
gag = as.matrix(transm.graph)
ga.graph = graph.edgelist(gag[,1:2])

V(ga.graph)$color <- "red"

# Transmission network
plot.igraph(ga.graph, edge.arrow.size=0, vertex.size=9,
            vertex.frame.color="black", vertex.label.color="black",
            vertex.label = V(ga.graph)$name,layout = layout_with_kk,
            vertex.label.cex=0.8, vertex.label.dist=0.0, edge.curved=0.0, asp=0, margin=0,
            main = "Transmission network of an infectious disease")

# Histogram of the network degree (how many newly individuals infected by a given node-individual)
trans.net <- ga.graph
hist(degree(trans.net), main = "Transmission histogram")



# 2. Simulate DNA evlution and construct a phylogenetic tree
#############################################################

# get the sequence of the pathogene you want to infect the individuals
# taken: HIV & flu
hiv.seq <- read_file("~/Dropbox/MPE_2017/HIV_K03455.txt")
flu.seq <- read_file("~/Dropbox/MPE_2017/flue_ZA_root.txt")
## Remove the break line in the string of DNA
clean.hiv.seq <-  gsub("\n", "", hiv.seq)
clean.flu.seq <-  gsub("\n", "", hiv.seq)

## For any part of the DNA you want to study its evolution,
# retrieve the range of interest.
## We downloaded the K03455, HIV-1 sequence with 9719 nucleotides

## Choose the gene of interest: env from 6172 to 8742 nucl position
hiv.seq.env <- substr(clean.hiv.seq, 6172,6180)

flu.seq.env <- substr(clean.hiv.seq, 190,200)



# Which evolutionary model of nucleotide subtitution: General Time Reversible (GTR)
# consider same Q matrix since all of them are same familly of virus
# Transmision rate anf nucleotide frequencies: the matrix Q
# with nucleotide substitution rate = (0.8 - 1.7)*10^-3 per site/year >>> 1.25*10^-3 (average)
proc <- GTR(rate.params = list("a"=0.00125, "b"=0.00125, "c"=0.00125,"d"=0.00125, "e"=0.00125, "f"=0.00125),
            base.freqs = c(0.3492804, 0.1695838, 0.2399844, 0.2411513)) # base frequencies come from above considered sequence

# Sequence as string
seedSeq <- hiv.seq.env

seedSeq.flu <- flu.seq.env

# Attach the process to the DNA strand of the root transmitter
nucleproc1 <- NucleotideSequence(string = seedSeq, processes = list(list(proc)))

# Attach the process to the DNA strand of the root transmitter
nucleproc2 <- NucleotideSequence(string = seedSeq.flu, processes = list(list(proc)))

# From the literature: nucleotide of HIV sequences are Gama-distributed
# with shape 0.9
# Sample rate multipliers from a discrete gamma distribution
# with 4 categories and shape parameter 0.5:
plusGamma(nucleproc1,proc,0.9) 
plusGamma(nucleproc2,proc,0.9) 

# Simulate the sequences
sim1 <- Simulate(PhyloSim(root.seq = nucleproc1,
                          phylo = transm.tree))

sim2 <- Simulate(PhyloSim(root.seq = nucleproc2,
                          phylo = transm.tree))

# # save the sequences with internal nodes which represent the recent common ancestor
# saveAlignment.PhyloSim(sim1, file = paste("HIV_simulation_node.fasta",sep=""))
# 
# # save the sequences without internal nodes which represent the recent common ancestor
# # this is the one we use to build phylogenetic tree
# saveAlignment.PhyloSim(sim1, file = paste("HIV_simulation.fasta",sep=""), skip.internal = TRUE, paranoid = TRUE)

# Plot the phylogenetic tree and the sequences together

postscript("Phylogenetic_tree_hiv.eps", width = 480, height = 480)
plot(sim1, num.pages = "auto", plot.ancestors = FALSE) # Make plot
dev.off()

postscript("Phylogenetic_tree_flu.eps", width = 480, height = 480)
plot(sim2, num.pages = "auto", plot.ancestors = FALSE) # Make plot
dev.off()
